@@ -1,6 +1,6 @@
 package Calendar::Slots;
 {
-  $Calendar::Slots::VERSION = '0.12';
+  $Calendar::Slots::VERSION = '0.14';
 }
 use Moose;
 use MooseX::AttributeHelpers;
@@ -29,12 +29,15 @@ has 'slots' => (
 
 sub slot {
     my $self = shift;
+    my @ret;
 	for my $slot ( $self->_create_slots( @_ ) ) {
 		$self->_validate($slot) unless $self->overlapping;
-		my @slots = $self->_merge( $slot, $self->all );
+		my @slots = $self->_merge( {}, $slot, $self->all );
 		$self->clear_slots;
 		$self->add_slots( @slots );
+        push @ret, @slots;
 	}
+    return @ret;
 }
 
 sub _create_slots {
@@ -80,29 +83,36 @@ sub _validate {
 #merge slots that are next to each other
 sub _merge {
     my $self = shift;
+    my $opts = shift;
     my $new  = shift;
-	my @slots = @_;
-	unless( scalar @slots ) {
-		return $new;
-	}
+    my @slots = @_;
+    unless( scalar @slots ) {
+        return $new;
+    }
     my $slot = shift @slots;
-	unless( $slot->same_type($new) && $slot->same_day($new) ) {
-		return ( $slot, $self->_merge( $new, @slots ) );
-	}
+    if( $opts->{materialize} ) {
+        if( ! $slot->same_weekday( $new ) ) {
+            return ( $slot, $self->_merge( $opts, $new, @slots ) );
+        }
+    }
+    elsif( !( $slot->same_type($new) && $slot->same_day($new) ) ) {
+        # skip this slot
+        return ( $slot, $self->_merge( $opts, $new, @slots ) );
+    }
     my ( $s1, $s2, $n1, $n2 ) = ( 
         $slot->start, $slot->end, $new->start, $new->end
     );
-    # warn join ';', $new->name, '--> ', $n1, $n2, '***', $slot->name, $s1, $s2;
+    #warn join ';', 'new', $new->name, ';', $n1, $n2, '***', $slot->name, $s1, $s2;
     if ( $slot->name eq $new->name ) {
         # s: 10-12, n: 09-12 => merge start
         if ( $n1 < $s1 and $n2 <= $s2 and $n2 >= $s1 ) {
             $slot->start( $new->start );
-            return $self->_merge( $slot, @slots );
+            return $self->_merge( $opts, $slot, @slots );
         }
         # s: 10-12, n: 11-13 => merge end
         elsif ( $n1 <= $s2 and $n1 >= $s1 and $n2 > $s2 ) {
             $slot->end( $new->end );
-            return $self->_merge( $slot, @slots );
+            return $self->_merge( $opts, $slot, @slots );
         }
         # s: 10-12, n: 11-12 => discard new
         elsif ( $n1 >= $s1 and $n2 <= $s2 ) {
@@ -112,27 +122,28 @@ sub _merge {
         elsif ( $n1 < $s1 and $s2 < $n2 ) {
             $slot->start( $new->start );
             $slot->end( $new->end );
-            return $self->_merge( $slot, @slots );
+            return $self->_merge( $opts, $slot, @slots );
         }
         # s: 10-12, n: 01-05 => add
-		else {
-			return ($slot, $self->_merge( $new, @slots) );
-		}
+        else {
+            return ($slot, $self->_merge( $opts, $new, @slots) );
+        }
     } elsif( ! $self->overlapping ) {
         if ( $slot->start == $new->start and $slot->end == $new->end ) {
-            return $self->_merge($new, @slots);
+            return $self->_merge($opts, $new, @slots);
         }
-        elsif ( $new->start < $slot->start and $new->end >= $slot->start and $new->end <= $slot->end ) {
+        elsif ( $n1 <= $s1 and ( $n2 >= $s1 and $n2 <= $s2 ) ) {
             $slot->start( $new->end );
-            return ($slot, $self->_merge( $new, @slots ) );
+            return ($slot, $self->_merge( $opts, $new, @slots ) );
         }
-        elsif ( $new->start >= $slot->start and $new->start < $slot->end and $new->end >= $slot->end ) {
+        elsif ( ( $n1 >= $s1 and $n1 < $s2 ) and $n2 >= $s2 ) {
             $slot->end( $new->start );
-            return ($slot, $self->_merge( $new, @slots ) );
+            return ($slot, $self->_merge( $opts, $new, @slots ) );
         }
         elsif ( $slot->start < $new->start and $new->end < $slot->end ) {
             my $third = new Calendar::Slots::Slot(
                 name  => $slot->name,
+                data  => $slot->data,
                 when   => $slot->when,
                 start => $new->end,
                 end   => $slot->end
@@ -141,18 +152,19 @@ sub _merge {
             return ( $slot, $third, $new, @slots );
         }
         elsif ( $new->start < $slot->start and $slot->end < $new->end ) {
-            return $self->_merge( $new, @slots );
+            return $self->_merge( $opts, $new, @slots );
         }
-		else {
-			return ($slot, $self->_merge( $new, @slots) );
-		}
-	}
-	else {
-		# overlapping 
-		return ($slot, $self->_merge( $new, @slots) );
-	}
+        else {
+            return ($slot, $self->_merge( $opts, $new, @slots) );
+        }
+    }
+    else {
+        # overlapping 
+        return ($slot, $self->_merge( $opts, $new, @slots) );
+    }
 }
 
+
 sub all {
 	my $self = shift;
 	@{ $self->slots };
@@ -165,6 +177,65 @@ sub sorted {
 	} $self->all;
 }
 
+sub clone {
+    my $self = shift;
+    my $new = __PACKAGE__->new( %$self, slots=>[] );
+    $new->clear_slots;
+    $new->slot( %$_ ) for $self->all;
+    return $new;
+}
+
+sub week_of {
+    my ($self, $date ) = @_;
+    $date =~ s/\D//g;
+
+    # clone 
+    $self = $self->clone;
+
+    # find a monday
+    my $dt = parse_dt( '%Y%m%d', $date );
+    my $wk = $dt->wday;
+    my $ep = $dt->epoch;
+    # sunday
+    my $sunday_ep = $ep + ( (7-$wk) * 86400 );
+    my $sunday = substr DateTime->from_epoch( epoch=>$sunday_ep ), 0, 10;
+    # monday
+    $wk = 7 if $wk == 0;
+    my $monday_ep = $ep - ( ( $wk - 1 ) * 86400 );
+    my $monday = substr DateTime->from_epoch( epoch=>$monday_ep ), 0, 10;
+    $sunday =~ s/\D//g;
+    $monday =~ s/\D//g;
+    #  die "$monday - $sunday";
+    sub _dump { require YAML; YAML::Dump( \@_ ) };
+    return $self->materialize( $monday, $sunday );
+}
+
+sub materialize {
+    my ($self, $start, $end) = @_;
+
+    # get rid of dates outside this date range
+    my @s_wk ;
+    my @s_date;
+    for my $slot ( $self->all ) {
+        if( $slot->type eq 'weekday' ) {
+            push @s_wk, $slot;
+        }
+        elsif( $start <= $slot->when && $slot->when <= $end ) {
+            push @s_date, $slot;
+        }
+    }
+
+    # merge materialized
+    $self->clear_slots;
+    my @slots = @s_wk;
+    for( @s_date ) {
+        @slots = $self->_merge( { materialize => 1 }, $_, @slots );  # put the dates first
+    }
+    $self->clear_slots;
+    $self->add_slots( @slots );
+    return $self;
+}
+
 sub find {
     my $self = shift;
 	my %args = @_;
@@ -183,6 +254,15 @@ sub name {
 	return $slot->name if $slot = find( @_ );
 }
 
+sub as_table {
+    my $self = shift;
+    require Data::Format::Pretty::Console;
+    return Data::Format::Pretty::Console::format_pretty( 
+        [ map { { %$_ } } $self->sorted ],
+        { table_column_orders=>[[qw/name start end when type _weekday data/]] }
+    );
+}
+
 1; 
 
 __END__
@@ -195,7 +275,7 @@ Calendar::Slots - Manage time slots
 
 =head1 VERSION
 
-version 0.12
+version 0.14
 
 =head1 SYNOPSIS
 
@@ -276,10 +356,35 @@ Shortcut method to L<find|/find> a slot and return a name.
 
 Returns a  ARRAY of all slot objects in the calendar.
 
+=head2 materialize ( start_date, end_date ) 
+
+Returns an instance of L<Calendar::Slots> with
+date slots converted into weekdays for a given
+date range.
+
+    my $new_cal = $cal->materialize( 2012_10_22, 2012_10_28 );
+
+=head2 week_of ( date ) 
+
+Returns a materialized instance of L<Calendar::Slots> with actual 
+dates merged for the week that comprises 
+the passed C<date>.  
+
+    my $week = $cal->week_of( 2012_10_22 );
+    $week->find( weekday=>2, time=>10_30 );  # ...
+
 =head2 all 
 
 Returns an ARRAY of all slot objects in the calendar.
 
+=head2 as_table
+
+Returns a console string as a table for the calendar.
+
+Requires that L<Data::Format::Pretty::Console> be installed.
+
+    print $cal->as_table;
+
 =head1 SEE ALSO
 
 L<DateTime::SpanSet>
